/*
 * Copyright 2010 Trustees of the University of Pennsylvania Licensed under the
 * Educational Community License, Version 2.0 (the "License"); you may
 * not use this file except in compliance with the License. You may
 * obtain a copy of the License at
 *
 * http://www.osedu.org/licenses/ECL-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an "AS IS"
 * BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * permissions and limitations under the License.
 */

import groovy.xml.MarkupBuilder
import groovy.grape.Grape
import org.codehaus.groovy.tools.GrapeMain

ant = new AntBuilder()

assert args: "a build step must be passed"

metridocVersion = System.getProperty("metridoc.version","[0.10,)")

root = new File(getClass().protectionDomain.codeSource.location.path).parent
target = "${root}/target"
reports = "${target}/reports"
targetClasses = "${target}/classes"
targetTestClasses = "${target}/testClasses"
targetJunit = "${target}/junit"
main = "${root}/src"
jobs = "${main}/jobs"
test = "${root}/test"
resources = "${root}/resources"
testResources = "${root}/testResources"
home = System.getProperty("user.home")
groovy = "${home}/.groovy"
grapeConfig = "${groovy}/grapeConfig.xml"
metridocProject = "http://metridoc.googlecode.com/svn/trunk/core/metridoc-project/src/main/groovy"
encoding = "UTF-8"
runUrl = new URL("${metridocProject}/run.groovy")
buildUrl = new URL("${metridocProject}/build.groovy")
jobUrl = new URL("${metridocProject}/jobs/fooJob.groovy")
logbackUrl = new URL("${metridocProject}/logback.groovy")
newLine = System.getProperty("line.separator")
tab = "    "
metridocLibraryConfig = "load !{user.home}/.metridoc/lib/*.jar"
metridocLibraryDirectory = "${home}/.metridoc/lib"

groovyStarterConf = "groovy.starter.conf"

if (args[0] == "junit") {
    return //doing a junit test, just load the binding
}

println "Running build with steps ${args}"

args.each {
    this."${it}"()
}

def createTarget() {
    ant.mkdir(dir: targetClasses)
}

def createTestTarget() {
    ant.mkdir(dir: targetTestClasses)
}

def clean() {
    ant.delete(dir: target)
}

def copyResources() {
    if (new File(resources).exists()) {
        ant.copy(todir: targetClasses) {
            fileset(dir: resources)
        }
    }
}

def copyTestResources() {
    if (new File(testResources).exists()) {
        ant.copy(todir: targetTestClasses) {
            fileset(dir: testResources)
        }
    }
}

def compile() {
    createTarget()
    copyResources()
    ant.taskdef name: "groovyc", classname: "org.codehaus.groovy.ant.Groovyc"
    ant.groovyc(srcdir: main, destdir: targetClasses)
}

def testCompile() {
    compile()
    createTestTarget()
    copyTestResources()
    ant.taskdef name: "groovyc", classname: "org.codehaus.groovy.ant.Groovyc"
    ant.groovyc(srcdir: test, destdir: targetTestClasses) {
        classpath {
            pathelement(location: targetClasses)
        }
    }
    ant.mkdir(dir: targetJunit)
}

def test() {
    testCompile()
    ant.junit(haltonfailure: 'yes') {
        classpath {
            pathelement(location: targetClasses)
            pathelement(location: targetTestClasses)
        }
        formatter(type: 'plain', usefile: 'false')
        batchtest(todir: targetJunit) {
            fileset(dir: targetTestClasses) {
                include(name: '**/*Test*')
                exclude(name: "**/*\$*") //exclude the compiled closures
            }
        }
    }
}

def createFolders() {
    ant.mkdir(dir: main)
    ant.mkdir(dir: jobs)
    ant.mkdir(dir: test)
    ant.mkdir(dir: testResources)
    ant.mkdir(dir: resources)
}



def createProject() {
    createGrapeConfigXml()
    installMetridoc()
    createFolders()
    createBasicJobs()
}

def reset() {
    resetGrape()
    resetMetridoc()
}

def resetMetridoc() {
    ant.delete(dir: metridocLibraryDirectory)
    ant.mkdir(dir: metridocLibraryDirectory)
}

def installMetridoc() {
    configGroovy()
    //System.setProperty('groovy.grape.report.downloads', 'true') //will report what is being resolved instead of hanging
    // set the instance so we can re-set the logger

    if(new File(grapeConfig).exists()) {
        System.setProperty("grape.config", grapeConfig)
    }
	def version = System.getProperty("metridoc.version", '*')
    println "using metridoc version ${version}"
	GrapeMain.main("install","com.googlecode.metridoc", "metridoc-all", version)
    def uris = Grape.resolve([:], [group: 'com.googlecode.metridoc', module: 'metridoc-all'])
    resetMetridoc()

    uris.each {URI uri ->
        def output
        def from = new File(uri)
        def bytes = from.bytes
        def to = new File("${metridocLibraryDirectory}/${from.name}")
        assert to.createNewFile(): "Could not create new file"
        println "copying ${from} to ${to}"
        to.append(from.bytes)
    }
}

/**
 * creates the groovy directory and a better grape xml file that includes maven central while excluding java.net
 * due to reliability
 */
def createGrapeConfigXml() {

    def grapeConfig = new File("${groovy}/grapeConfig.xml")

    if (grapeConfig.exists()) return


    ant.mkdir(dir: groovy)
    def writer = new FileWriter(grapeConfig)
    def xml = new MarkupBuilder(writer)

    //local->sonatype_snapshot->central->codehaus->ibiblio
    xml.ivysettings() {
        def defaultResolver = "downloadGrapes"
        settings(defaultResolver: defaultResolver)
        resolvers() {
            chain(name: defaultResolver) {
                filesystem(name: "cachedGrapes") {
                    ivy(pattern: "${groovy}/grapes/[organisation]/[module]/ivy-[revision].xml")
                    artifact(pattern: "${groovy}/grapes/[organisation]/[module]/[type]s/[artifact]-[revision].[ext]")
                }

                ibiblio(name: "sonatype_snapshot", root: "https://oss.sonatype.org/content/repositories/snapshots", m2compatible: "true")
                ibiblio(name: "central", m2compatible: "true")
                ibiblio(name: "codehaus", root: "http://repository.codehaus.org/", m2compatible: "true")
                ibiblio(name: "ibiblio", m2compatible: "true")
            }
        }
    }
}

def createBasicJobs() {
    grabAndWrite(runUrl, "${root}/run.groovy")
    grabAndWrite(jobUrl, "${jobs}/fooJob.groovy")
    grabAndWrite(logbackUrl, "${resources}/logback.groovy")

}

def resetGrape() {
    ant.delete(dir: "${groovy}/grapes")
    createGrapeConfigXml()
}

def upgrade() {
    reset()
    createFolders()
    grabAndWrite(runUrl, "${root}/run.groovy")
    grabAndWrite(buildUrl, "${root}/build.groovy")
    def logback = "${resources}/logback.groovy"
    if (!(new File(logback).exists())) {
        grabAndWrite(logbackUrl, logback)
    }
    installMetridoc()
}

def configGroovy() {
    def configFilePath = System.getProperty(groovyStarterConf)
    def configFile

    configFile = new File(configFilePath)
    if (!configFile) {
        def message = "the system property groovy.starter.conf could not be found. The property gives the " +
                "location of the groovy config file that indicates which libraries to load"

        throw new FileNotFoundException(message)
    }

    if (!fileHasMetridocLibrary(configFile)) {
        appendLibRefToConfig(configFile)
    }
}

private boolean fileHasMetridocLibrary(File file) {
    def result = false
    file.eachLine {String line ->
        if (line.contains(metridocLibraryConfig)) {
            result = true
        }
    }

    return result
}

private void appendLibRefToConfig(File file) {
    def writer = new FileWriter(file, true)
    writer.println()
    writer.println("${tab}# load metridoc libraries")
    writer.println("${tab}${metridocLibraryConfig}")
    writer.close()
}

private grabAndWrite(URL from, String to) {

    def stream
    try {
        def urlConnection = from.openConnection()
        stream = from.openStream()
        def content = stream.getText("UTF-8")
        new File(to).write(content, encoding)
    } finally {
        try {
            stream.close()
        } catch (Exception ex) {}
    }
}